import { prop, unless, both, always, curry } from 'ramda';
import { dispatch, isString, isNotEmpty, isFunction } from 'ramda-extension';
import React from 'react';

/**
 * Returns the display name of a React component, falling back appropriately.
 *
 * @param {React.Component} Component React component to get the name of
 * @returns {string}
 */

var getDisplayName = dispatch([prop('displayName'), prop('name'), unless(both(isString, isNotEmpty), always('Component'))]);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var withProps = curry(function (otherProps, NextComponent) {
  var WithProps = function WithProps(props) {
    return React.createElement(NextComponent, _extends({}, props, isFunction(otherProps) ? otherProps(props) : otherProps));
  };

  WithProps.displayName = "WithProps(" + getDisplayName(NextComponent) + ")";
  return WithProps;
});

export { getDisplayName, withProps };
